/*
 * AbstractTypeUser.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/AbstractTypeUser.h>
#include <llvm/DerivedTypes.h>
#include <llvm/Type.h>
#include <llvm/Value.h>

using namespace boost::python;

namespace llvm { namespace Python {

void wrap_AbstractTypeUser()
{
    class_<AbstractTypeUser, boost::noncopyable>("AbstractTypeUser", no_init)

        .def("refineAbstractType", &AbstractTypeUser::refineAbstractType)
        .def("typeBecameConcrete", &AbstractTypeUser::typeBecameConcrete)
        .def("dump", &AbstractTypeUser::dump)
        ;

    class_<PATypeHandle>("PATypeHandle", init<const Type*, AbstractTypeUser*>())
        // additional constructors
        .def(init<const PATypeHandle&>())

        .def("get", &PATypeHandle::get,
             return_internal_reference<>())
        .def(self == other<const Type*>())
        .def("removeUserFromConcrete", &PATypeHandle::removeUserFromConcrete)
        ;

    // Wrap PATypeHandle::operator Type*()
    implicitly_convertible<PATypeHandle, Type*>();
    
    class_<PATypeHolder>("PATypeHolder", init<const Type*>())
        // additional constructors
        .def(init<const PATypeHolder&>())

        .def("get", &PATypeHolder::get,
             return_internal_reference<>())
        .def("getRawType", &PATypeHolder::getRawType,
             return_internal_reference<>())
        ;
    
    // Wrap PATypeHolder::operator Type*()
    implicitly_convertible<PATypeHolder, Type*>();
}

}} // namespace llvm::Python

// EOF
