/*
 * Analysis_Verifier.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Analysis/Verifier.h>
#include <llvm/Function.h>
#include <llvm/Module.h>
#include <llvm/Pass.h>

using namespace boost::python;

namespace llvm { namespace Python {

BOOST_PYTHON_FUNCTION_OVERLOADS(createVerifierPass_overloads, createVerifierPass, 0, 1)
BOOST_PYTHON_FUNCTION_OVERLOADS(verifyModule_overloads, verifyModule, 1, 2)
BOOST_PYTHON_FUNCTION_OVERLOADS(verifyFunction_overloads, verifyFunction, 1, 2)

void wrap_Analysis_Verifier()
{
    enum_<VerifierFailureAction>("VerifierFailureAction")
        .value("AbortProcessAction", AbortProcessAction)
        .value("ThrowExceptionAction", ThrowExceptionAction)
        .value("PrintMessageAction", PrintMessageAction)
        .value("ReturnStatusAction", ReturnStatusAction)
        ;

    def("createVerifierPass",
        createVerifierPass,
        createVerifierPass_overloads()
            [return_value_policy<manage_new_object>()]);
    def("verifyModule",
        verifyModule,
        verifyModule_overloads());
    def("verifyFunction",
        verifyFunction,
        verifyFunction_overloads());
}

}} // namespace llvm::Python

// EOF

