/*
 * Annotation.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Support/Annotation.h>

using namespace boost::python;

namespace llvm { namespace Python {

void wrap_Annotation()
{
    class_<AnnotationID, boost::noncopyable>
        ("AnnotationID", no_init)
        .def(self == other<const AnnotationID>())
        .def(self < other<const AnnotationID>())
        ;

    class_<Annotation, boost::noncopyable>("Annotation", init<AnnotationID>())
        .def("getID", &Annotation::getID)
        .def("getNext", &Annotation::getNext,
             return_internal_reference<>())
        ;

    class_<Annotable, boost::noncopyable>("Annotable")
        .def("getAnnotation", &Annotable::getAnnotation,
             return_internal_reference<>())
        .def("getOrCreateAnnotation", &Annotable::getOrCreateAnnotation,
             return_internal_reference<>())
        .def("addAnnotation", &Annotable::addAnnotation)
        .def("unlinkAnnotation", &Annotable::unlinkAnnotation,
             return_value_policy<manage_new_object>())
        .def("deleteAnnotation", &Annotable::deleteAnnotation)
        ;

    AnnotationID (*getID)(const std::string&)
        = &AnnotationManager::getID;
    AnnotationID (*getID2)(const std::string&, AnnotationManager::Factory, void*)
        = &AnnotationManager::getID;
    
    class_<AnnotationManager, boost::noncopyable>("AnnotationManager", no_init)
        .def("getID", getID)
        .def("getID", getID2)
        .staticmethod("getID")
        .def("getName", &AnnotationManager::getName,
             return_internal_reference<>())
        .staticmethod("getName")
        .def("registerAnnotationFactory", &AnnotationManager::registerAnnotationFactory)
        .staticmethod("registerAnnotationFactory")
        .def("createAnnotation", &AnnotationManager::createAnnotation,
             return_value_policy<manage_new_object>())
        .staticmethod("createAnnotation")
        ;
    
}
}} // namespace llvm::Python

// EOF
