/*
 * Argument.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Argument.h>
#include <llvm/Function.h>
#include <llvm/SymbolTable.h>
#include <llvm/Type.h>

using namespace boost::python;

#include "inst_init.h"

namespace llvm { namespace Python {

BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Argument_overloads, setName, 1, 2)

template Argument* InstInitFn3(const Type*, const std::string&, Function*);

void wrap_Argument()
{
    // Overloaded functions need some C++ preamble
    const Function* (Argument::*getParent)() const = &Argument::getParent;
    const Argument* (Argument::*getNext)() const = &Argument::getNext;
    const Argument* (Argument::*getPrev)() const = &Argument::getPrev;
    bool (*classof_Argument)(const Argument*) = &Argument::classof;
    bool (*classof_Value)(const Value*) = &Argument::classof;

    class_<Argument, boost::noncopyable, bases<Value> >
        ("Argument", init<const Type*, optional<std::string> >())
        // ownership transmits to Function*
        .def("init",
             InstInitFn3<Argument, const Type*, const std::string&, Function*>,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<3, 0> >())
        .staticmethod("init")

        .def("setName", &Argument::setName, Argument_overloads())
        .def("getParent", getParent,
             return_internal_reference<>())
        .def("getNext", getNext,
             return_internal_reference<>())
        .def("getPrev", getPrev,
             return_internal_reference<>())
        .def("print_", &Argument::print)
        .def("classof", classof_Argument)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}
}} // namespace llvm::Python

// EOF
