/*
 * BasicBlock.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Assembly/AsmAnnotationWriter.h>
#include <llvm/BasicBlock.h>
#include <llvm/Function.h>
#include <llvm/InstrTypes.h> // TerminatorInst
#include <llvm/SymbolTable.h>

#include "scitbx/container_conversions.h"

using namespace boost::python;
using namespace scitbx::boost_python::container_conversions;

#include "inst_init.h"

namespace llvm { namespace Python {

template BasicBlock* InstInitFn2(std::string, Function*);
template BasicBlock* InstInitFn3(std::string, Function*, BasicBlock*);

void wrap_BasicBlock()
{
    // Overloaded functions need some C++ preamble
    const Function* (BasicBlock::*getParent)() const = &BasicBlock::getParent;
    const BasicBlock* (BasicBlock::*getNext)() const = &BasicBlock::getNext;
    const BasicBlock* (BasicBlock::*getPrev)() const = &BasicBlock::getPrev;
    const TerminatorInst* const (BasicBlock::*getTerminator)() const = &BasicBlock::getTerminator;

    BasicBlock::const_iterator          (BasicBlock::*begin)() const =  &BasicBlock::begin;
    BasicBlock::const_iterator          (BasicBlock::*end)  () const = &BasicBlock::end;
    BasicBlock::const_reverse_iterator (BasicBlock::*rbegin)() const = &BasicBlock::rbegin;
    BasicBlock::const_reverse_iterator (BasicBlock::*rend)  () const = &BasicBlock::rend;
    const Instruction& (BasicBlock::*front)() const = &BasicBlock::front;
    const Instruction& (BasicBlock::*back)() const = &BasicBlock::back;
    const BasicBlock::InstListType& (BasicBlock::*getInstList)() const = &BasicBlock::getInstList;
    void (BasicBlock::*print)(std::ostream&) const = &BasicBlock::print;
    void (BasicBlock::*print_AAW)(std::ostream&, AssemblyAnnotationWriter*) const = &BasicBlock::print;
    bool (*classof_BasicBlock)(const BasicBlock*) = &BasicBlock::classof;
    bool (*classof_Value)(const Value*) = &BasicBlock::classof;

    class_<BasicBlock, boost::noncopyable, bases<Value> >
        ("BasicBlock", init<optional<std::string> >())
        // Ownership transmits to Function
        .def("init", InstInitFn2<BasicBlock, const std::string, Function*>,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<2, 0> >())
        // Ownership transmits to Function
        .def("init", InstInitFn3<BasicBlock, const std::string, Function*, BasicBlock*>,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<2, 0> >())
        .staticmethod("init")

        .def("setName", &BasicBlock::setName)
        .def("getParent", getParent,
             return_internal_reference<>())
        .def("getNext", getNext,
             return_internal_reference<>())
        .def("getPrev", getPrev,
             return_internal_reference<>())
        .def("getTerminator", getTerminator,
             return_internal_reference<>())
        .def("removeFromParent", &BasicBlock::removeFromParent)
        .def("eraseFromParent", &BasicBlock::eraseFromParent)
        .add_property("instructions", range(begin, end))
        //.def("begin", begin)
        //.def("end", end)
        .add_property("reverse_instructions", range(rbegin, rend))
        //.def("rbegin", rbegin)
        //.def("rend", rend)
        .def("size", &BasicBlock::size)
        .def("empty", &BasicBlock::empty)
        .def("front", front,
             return_internal_reference<>())
        .def("back", back,
             return_internal_reference<>())
        .def("getInstList", getInstList,
             return_internal_reference<>())
        .def("print_", print)
        .def("print_", print_AAW)
        .def("classof", classof_BasicBlock)
        .def("classof", classof_Value)
        .staticmethod("classof")
        .def("dropAllReferences", &BasicBlock::dropAllReferences)
        .def("removePredecessor", &BasicBlock::removePredecessor)
        .def("splitBasicBlock", &BasicBlock::splitBasicBlock,
             return_internal_reference<>())
        ;

    // Register tuple conversion for iplist<Instruction> return values
    to_python_converter<iplist<Instruction>,
                        to_tuple<iplist<Instruction> > >();
}
}} // namespace llvm::Python

// EOF
