/*
 * Bytecode_Reader.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Bytecode/Reader.h>
#include <llvm/Bytecode/BytecodeHandler.h>

using namespace boost::python;

namespace llvm { namespace Python {

BOOST_PYTHON_FUNCTION_OVERLOADS(getBytecodeModuleProvider_overloads, getBytecodeModuleProvider, 1, 2)
BOOST_PYTHON_FUNCTION_OVERLOADS(getBytecodeBufferModuleProvider_overloads, getBytecodeBufferModuleProvider, 2, 4)
BOOST_PYTHON_FUNCTION_OVERLOADS(ParseBytecodeFile_overloads, ParseBytecodeFile, 1, 2)
BOOST_PYTHON_FUNCTION_OVERLOADS(ParseBytecodeBuffer_overloads, ParseBytecodeBuffer, 2, 4)

void wrap_Bytecode_Reader()
{
    // Name C++ overloads
    bool (*GetBytecodeSymbols1)(const sys::Path&, std::vector<std::string>&)
        = GetBytecodeSymbols;
    ModuleProvider* (*GetBytecodeSymbols2)(const unsigned char*, unsigned, const std::string&, std::vector<std::string>&)
        = GetBytecodeSymbols;

    def("getBytecodeModuleProvider", getBytecodeModuleProvider,
        getBytecodeModuleProvider_overloads()
        [return_value_policy<manage_new_object>()]);
    def("getBytecodeBufferModuleProvider", getBytecodeBufferModuleProvider,
        getBytecodeBufferModuleProvider_overloads()
        [return_value_policy<manage_new_object>()]);
    def("ParseBytecodeFile", ParseBytecodeFile,
        ParseBytecodeFile_overloads()
        [return_value_policy<manage_new_object>()]);
    def("ParseBytecodeBuffer", ParseBytecodeBuffer,
        ParseBytecodeBuffer_overloads()
        [return_value_policy<manage_new_object>()]);
    def("GetBytecodeDependentLibraries", GetBytecodeDependentLibraries);
    def("GetBytecodeSymbols", GetBytecodeSymbols1);
    def("GetBytecodeSymbols", GetBytecodeSymbols2,
         return_value_policy<manage_new_object>());
}

}} // namespace llvm::Python

// EOF
