/*
 * CallGraphSCCPass.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/CallGraphSCCPass.h>
#include <llvm/Analysis/CallGraph.h>

#include "scitbx/container_conversions.h"

using namespace boost::python;
using namespace scitbx::boost_python::container_conversions;

namespace llvm { namespace Python {

void wrap_CallGraphSCCPass()
{
    // FIXME make overridable in Python
    class_<CallGraphSCCPass, boost::noncopyable, bases<ModulePass> >
        ("CallGraphSCCPass", no_init)

        .def("doInitialization", &CallGraphSCCPass::getPassName)
        .def("runOnSCC", &CallGraphSCCPass::getPassName)
        .def("doFinalization", &CallGraphSCCPass::getPassName)
        .def("runOnModule", &CallGraphSCCPass::getPassName)
        .def("getAnalysisUsage", &CallGraphSCCPass::getPassName)
        ;

    // Register tuple conversion for vector<CallGraphNode*> arguments
    from_python_sequence<std::vector<CallGraphNode*>,
                         variable_capacity_policy>();
}

}} // namespace llvm::Python

// EOF
