/*
 * Constant.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Constant.h>
#include <llvm/GlobalValue.h>
#include <llvm/SymbolTable.h>
#include <llvm/Type.h>

using namespace boost::python;

namespace llvm { namespace Python {

void wrap_Constant()
{
    // Overloaded functions need some C++ preamble
    const Constant* (Constant::*getOperand)(unsigned) const = &Constant::getOperand;
    bool (*classof_Constant)(const Constant*) = &Constant::classof;
    bool (*classof_GlobalValue)(const GlobalValue*) = &Constant::classof;
    bool (*classof_Value)(const Value*) = &Constant::classof;

    class_<Constant, boost::noncopyable, bases<User> >("Constant", no_init)

        .def("setName", &Constant::setName)
        .def("getNullValue", &Constant::getNullValue,
             return_value_policy<reference_existing_object>())
        .staticmethod("getNullValue")
        .def("isNullValue", &Constant::isNullValue)
        .def("print_", &Constant::print)
        .def("getOperand", getOperand,
             return_value_policy<reference_existing_object>())
        .def("setOperand", &Constant::setOperand)
        .def("destroyConstant", &Constant::destroyConstant)
        .def("classof", classof_Constant)
        .def("classof", classof_GlobalValue)
        .def("classof", classof_Value)
        .staticmethod("classof")
        .def("replaceUsesOfWithOnConstant", &Constant::replaceUsesOfWithOnConstant)
        .def("clearAllValueMaps", &Constant::clearAllValueMaps)
        .staticmethod("clearAllValueMaps")
        ;
}
}} // namespace llvm::Python

// EOF
