/*
 * ExecutionEngine.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <algorithm>
#include <vector>
#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/CodeGen/IntrinsicLowering.h>
#include <llvm/Constant.h>
#include <llvm/ExecutionEngine/ExecutionEngine.h>
#include <llvm/ExecutionEngine/GenericValue.h>
#include <llvm/Function.h>
#include <llvm/GlobalValue.h>
#include <llvm/GlobalVariable.h>
#include <llvm/Module.h>
#include <llvm/ModuleProvider.h>
#include <llvm/Target/TargetData.h>
#include <llvm/Type.h>
#include "GenericValueHolder.h"

using namespace boost::python;

//FIXME doesn't work
//BOOST_PYTHON_OPAQUE_SPECIALIZED_TYPE_ID(void)

namespace llvm { namespace Python {

BOOST_PYTHON_FUNCTION_OVERLOADS(create_overloads, ExecutionEngine::create, 2, 3)

/*
GenericValueHolder runFunction(ExecutionEngine* EE,
                               Function* F,
                               const std::vector<GenericValueHolder>& GVHArgs)
{
    std::vector<GenericValue> GVArgs(GVHArgs.size());
    std::copy(GVHArgs.begin(), GVHArgs.end(), GVArgs.begin());
    return EE->runFunction(F, GVArgs);
}
*/

GenericValueHolder runFunction(ExecutionEngine& ee,
                               Function* F,
                               tuple Args)
{
    int nArgs = extract<int>(Args.attr("__len__")());
    std::vector<GenericValue> GVArgs(nArgs);
    for (int idx = 0; idx < nArgs; idx++) {
        // can raise Python TypeError
        GenericValueHolder gv = extract<const GenericValueHolder&>(Args[idx]);
        GVArgs[idx] = gv;
    }
    return ee.runFunction(F, GVArgs);
}

void StoreValueToMemory(ExecutionEngine& ee,
                        GenericValueHolder Val,
                        void* Ptr,
                        const Type* Ty)
{
    ee.StoreValueToMemory(Val, reinterpret_cast<GenericValue*>(Ptr), Ty);
}

void wrap_ExecutionEngine()
{
    class_<ExecutionEngine, boost::noncopyable>("ExecutionEngine", no_init)
        //.def(init<ModuleProvider*>())
        //.def(init<Module*>())

        .def("getModule", &ExecutionEngine::getModule,
             return_internal_reference<>())
        .def("getTargetData", &ExecutionEngine::getTargetData,
             return_internal_reference<>())
        .def("create", &ExecutionEngine::create,
             create_overloads()
             [return_value_policy<manage_new_object,
                 with_custodian_and_ward_postcall<0, 1> >()])
        .staticmethod("create")
        .def("runFunction", runFunction)
        .def("runFunctionAsMain", &ExecutionEngine::runFunctionAsMain)
        .def("addGlobalMapping", &ExecutionEngine::addGlobalMapping)
        .def("updateGlobalMapping", &ExecutionEngine::updateGlobalMapping)
        //.def("getPointerToGlobalIfAvailable", &ExecutionEngine::getPointerToGlobalIfAvailable,
        //     return_value_policy<return_opaque_pointer>())
        //.def("getPointerToGlobal", &ExecutionEngine::getPointerToGlobal,
        //     return_value_policy<return_opaque_pointer>())
        //.def("getPointerToFunction", &ExecutionEngine::getPointerToFunction,
        //     return_value_policy<return_opaque_pointer>())
        //.def("getPointerToFunctionOrStub", &ExecutionEngine::getPointerToFunctionOrStub,
        //     return_value_policy<return_opaque_pointer>())
        .def("getGlobalValueAtAddress", &ExecutionEngine::getGlobalValueAtAddress,
             return_internal_reference<>())
        .def("StoreValueToMemory", StoreValueToMemory)
        .def("InitializeMemory", &ExecutionEngine::InitializeMemory)
        //.def("recompileAndRelinkFunction", &ExecutionEngine::recompileAndRelinkFunction,
        //     return_value_policy<return_opaque_pointer>())
        .def("freeMachineCodeForFunction", &ExecutionEngine::freeMachineCodeForFunction)
        //.def("getOrEmitGlobalVariable", &ExecutionEngine::getOrEmitGlobalVariable,
        //     return_value_policy<return_opaque_pointer>())
        ;
}

}} // namespace llvm::Python

// EOF

