/* -*-c++-*-
 * GenericValueHolder.h: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef LLVM_PYTHON_GENERICVALUEHOLDER_H
#define LLVM_PYTHON_GENERICVALUEHOLDER_H

#include <llvm/ExecutionEngine/GenericValue.h>

namespace llvm { namespace Python {

class GenericValueHolder
{
    union GenericValue GV;

public:
    GenericValueHolder()
        : GV()
    {}

    GenericValueHolder(const GenericValue& aGV)
        : GV(aGV)
    {}

    // CopyConstructible
    GenericValueHolder(const GenericValueHolder& aGVH)
        : GV(aGVH.GV)
    {}

    // Assignable
    GenericValueHolder& operator=(const GenericValueHolder& aGVH)
    {
        GV = aGVH.GV;
        return *this;
    }

    // EqualityComparable
    bool operator==(const GenericValueHolder& aGVH)
    {
        assert(sizeof(GenericValue) == sizeof(GV.LongVal));
        return (GV.LongVal == aGVH.GV.LongVal);
    }

    operator GenericValue&() { return GV; }
    operator const GenericValue&() const { return GV; }

    bool get_BoolVal() const { return GV.BoolVal; }
    void set_BoolVal(bool value) { GV.BoolVal = value; }
    unsigned char   get_UByteVal() const { return GV.UByteVal; }
    void set_UByteVal(unsigned char value) { GV.UByteVal = value; }
    signed   char   get_SByteVal() const { return GV.SByteVal; }
    void set_SByteVal(signed char value) { GV.SByteVal = value; }
    unsigned short  get_UShortVal() const { return GV.UShortVal; }
    void set_UShortVal(unsigned short value) { GV.UShortVal = value; }
    signed   short  get_ShortVal() const { return GV.ShortVal; }
    void set_ShortVal(signed short value) { GV.ShortVal = value; }
    unsigned int    get_UIntVal() const { return GV.UIntVal; }
    void set_UIntVal(unsigned int value) { GV.UIntVal = value; }
    signed   int    get_IntVal() const { return GV.IntVal; }
    void set_IntVal(signed int value) { GV.IntVal = value; }
    uint64_t        get_ULongVal() const { return GV.ULongVal; }
    void set_ULongVal(uint64_t value) { GV.ULongVal = value; }
    int64_t         get_LongVal() const { return GV.LongVal; }
    void set_LongVal(int64_t value) { GV.LongVal = value; }
    double          get_DoubleVal() const { return GV.DoubleVal; }
    void set_DoubleVal(double value) { GV.DoubleVal = value; }
    float           get_FloatVal() const { return GV.FloatVal; }
    void set_FloatVal(float value) { GV.FloatVal = value; }
    //FIXME wrap UIntPairVal (anonymous struct)
    PointerTy       get_PointerVal() const { return GV.PointerVal; }
    void set_PointerVal(PointerTy  value) { GV.PointerVal = value; }
};

}} // namespace llvm::Python

#endif //LLVM_PYTHON_GENERICVALUEHOLDER_H

// EOF
