/*
 * GlobalValue.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/DerivedTypes.h>
#include <llvm/GlobalValue.h>
#include <llvm/Module.h>

using namespace boost::python;

namespace llvm { namespace Python {

void wrap_GlobalValue()
{
    // Overloaded functions need some C++ preamble
    Module* (GlobalValue::*getParent)() = &GlobalValue::getParent;
    const Module* (GlobalValue::*getParent_const)() const = &GlobalValue::getParent;
    bool (*classof_GlobalValue)(const GlobalValue*) = &GlobalValue::classof;
    bool (*classof_Value)(const Value*) = &GlobalValue::classof;

    scope GlobalValueScope
        = class_<GlobalValue, boost::noncopyable, bases<Constant> >("GlobalValue", no_init)

        //FIXME undefined symbol: _ZN4llvm11GlobalValue26use_empty_except_constantsEv
        //.def("use_empty_except_constants", &GlobalValue::use_empty_except_constants)
        .def("getType", &GlobalValue::getType,
             return_internal_reference<>())
        .def("hasExternalLinkage", &GlobalValue::hasExternalLinkage)
        .def("hasLinkOnceLinkage", &GlobalValue::hasLinkOnceLinkage)
        .def("hasWeakLinkage", &GlobalValue::hasWeakLinkage)
        .def("hasAppendingLinkage", &GlobalValue::hasAppendingLinkage)
        .def("hasInternalLinkage", &GlobalValue::hasInternalLinkage)
        .def("setLinkage", &GlobalValue::setLinkage)
        .def("getLinkage", &GlobalValue::getLinkage)
        .def("hasNotBeenReadFromBytecode", &GlobalValue::hasNotBeenReadFromBytecode)
        .def("isNullValue", &GlobalValue::isNullValue)
        .def("destroyConstant", &GlobalValue::destroyConstant)
        .def("isExternal", &GlobalValue::isExternal)
        .def("getParent", getParent,
             return_internal_reference<>())
        .def("getParent", getParent_const,
             return_internal_reference<>())
        .def("removeDeadConstantUsers", &GlobalValue::removeDeadConstantUsers)
        .def("classof", classof_GlobalValue)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;

    enum_<GlobalValue::LinkageTypes>("LinkageTypes")
        .value("ExternalLinkage", GlobalValue::ExternalLinkage)
        .value("LinkOnceLinkage", GlobalValue::LinkOnceLinkage)
        .value("WeakLinkage", GlobalValue::WeakLinkage)
        .value("AppendingLinkage", GlobalValue::AppendingLinkage)
        .value("InternalLinkage", GlobalValue::InternalLinkage)
        .value("GhostLinkage", GlobalValue::GhostLinkage)
        ;
}
}} // namespace llvm::Python

// EOF
