/*
 * GlobalVariable.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/GlobalVariable.h>
#include <llvm/Module.h>
#include <llvm/SymbolTable.h>
#include <llvm/Type.h>

using namespace boost::python;

#include "inst_init.h"

namespace llvm { namespace Python {

BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(GlobalVariable_overloads1, setName, 1, 2)
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(GlobalVariable_overloads2, replaceUsesOfWithOnConstant, 2, 3)

template GlobalVariable* InstInitFn6(const Type*, bool, GlobalValue::LinkageTypes,
                                     Constant*, std::string, Module*);

void wrap_GlobalVariable()
{
    // Name C++ overloads
    Constant* (GlobalVariable::*getInitializer)() const = &GlobalVariable::getInitializer;
    const GlobalVariable* (GlobalVariable::*getNext)() const = &GlobalVariable::getNext;
    const GlobalVariable* (GlobalVariable::*getPrev)() const = &GlobalVariable::getPrev;
    bool (*classof_GlobalVariable)(const GlobalVariable*) = &GlobalVariable::classof;
    bool (*classof_Value)(const Value*) = &GlobalVariable::classof;

    class_<GlobalVariable, boost::noncopyable, bases<GlobalValue> >
        ("GlobalVariable",
         init<const Type*, bool, GlobalValue::LinkageTypes,
              optional<Constant*, std::string> >())
        // Ownership transmits to Module
        .def("init", InstInitFn6<GlobalVariable, const Type*, bool,
                                 GlobalValue::LinkageTypes,
                                 Constant*, std::string, Module*>,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<6, 0> >())

        .def("setName", &GlobalVariable::setName, GlobalVariable_overloads1())
        .def("isExternal", &GlobalVariable::isExternal)
        .def("hasInitializer", &GlobalVariable::hasInitializer)
        .def("getInitializer", getInitializer,
             return_value_policy<reference_existing_object>())
        .def("setInitializer", &GlobalVariable::setInitializer)
        .def("getNext", getNext,
             return_internal_reference<>())
        .def("getPrev", getPrev,
             return_internal_reference<>())
        .def("isConstant", &GlobalVariable::isConstant)
        .def("setConstant", &GlobalVariable::setConstant)
        .def("removeFromParent", &GlobalVariable::removeFromParent)
        .def("eraseFromParent", &GlobalVariable::eraseFromParent)
        .def("replaceUsesOfWithOnConstant",
             &GlobalVariable::replaceUsesOfWithOnConstant,
             GlobalVariable_overloads2())
        .def("print_", &GlobalVariable::print)
        .def("classof", classof_GlobalVariable)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}
}} // namespace llvm::Python

// EOF
