/*
 * InstrTypes.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Instruction.h>
#include <llvm/InstrTypes.h>
#include <llvm/Value.h>
#include <llvm/BasicBlock.h>

using namespace boost::python;

namespace llvm { namespace Python {

void wrap_TerminatorInst()
{
    // Name C++ overloads
    BasicBlock* (TerminatorInst::*getSuccessor)(unsigned) const = &TerminatorInst::getSuccessor;
    bool (*classof_TerminatorInst)(const TerminatorInst*) = &TerminatorInst::classof;
    bool (*classof_Instruction)(const Instruction*) = &TerminatorInst::classof;
    bool (*classof_Value)(const Value*) = &TerminatorInst::classof;

    class_<TerminatorInst, boost::noncopyable, bases<Instruction> >("TerminatorInst", no_init)
        .def("clone", &TerminatorInst::clone,
             return_value_policy<manage_new_object>())
        .def("getSuccessor", getSuccessor,
             return_internal_reference<>())
        .def("getNumSuccessors", &TerminatorInst::getNumSuccessors)
        .def("setSuccessor", &TerminatorInst::setSuccessor)
        .def("classof", classof_TerminatorInst)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_BinaryOperator()
{
    // Name C++ overloads
    bool (*classof_BinaryOperator)(const BinaryOperator*) = &BinaryOperator::classof;
    bool (*classof_Instruction)(const Instruction*) = &BinaryOperator::classof;
    bool (*classof_Value)(const Value*) = &BinaryOperator::classof;

    BinaryOperator* (*create1)(Instruction::BinaryOps, Value*, Value*, const std::string&, Instruction*)
        = &BinaryOperator::create;
    BinaryOperator* (*create2)(Instruction::BinaryOps, Value*, Value*, const std::string&, BasicBlock*)
        = &BinaryOperator::create;
    BinaryOperator* (*createAdd1)(Value*, Value*, const std::string&) = &BinaryOperator::createAdd;
    BinaryOperator* (*createAdd2)(Value*, Value*, const std::string&, BasicBlock*) = &BinaryOperator::createAdd;
    BinaryOperator* (*createAdd3)(Value*, Value*, const std::string&, Instruction*) = &BinaryOperator::createAdd;
    BinaryOperator* (*createSub1)(Value*, Value*, const std::string&) = &BinaryOperator::createSub;
    BinaryOperator* (*createSub2)(Value*, Value*, const std::string&, BasicBlock*) = &BinaryOperator::createSub;
    BinaryOperator* (*createSub3)(Value*, Value*, const std::string&, Instruction*) = &BinaryOperator::createSub;
    BinaryOperator* (*createMul1)(Value*, Value*, const std::string&) = &BinaryOperator::createMul;
    BinaryOperator* (*createMul2)(Value*, Value*, const std::string&, BasicBlock*) = &BinaryOperator::createMul;
    BinaryOperator* (*createMul3)(Value*, Value*, const std::string&, Instruction*) = &BinaryOperator::createMul;
    BinaryOperator* (*createDiv1)(Value*, Value*, const std::string&) = &BinaryOperator::createDiv;
    BinaryOperator* (*createDiv2)(Value*, Value*, const std::string&, BasicBlock*) = &BinaryOperator::createDiv;
    BinaryOperator* (*createDiv3)(Value*, Value*, const std::string&, Instruction*) = &BinaryOperator::createDiv;
    BinaryOperator* (*createRem1)(Value*, Value*, const std::string&) = &BinaryOperator::createRem;
    BinaryOperator* (*createRem2)(Value*, Value*, const std::string&, BasicBlock*) = &BinaryOperator::createRem;
    BinaryOperator* (*createRem3)(Value*, Value*, const std::string&, Instruction*) = &BinaryOperator::createRem;
    BinaryOperator* (*createAnd1)(Value*, Value*, const std::string&) = &BinaryOperator::createAnd;
    BinaryOperator* (*createAnd2)(Value*, Value*, const std::string&, BasicBlock*) = &BinaryOperator::createAnd;
    BinaryOperator* (*createAnd3)(Value*, Value*, const std::string&, Instruction*) = &BinaryOperator::createAnd;
    BinaryOperator* (*createOr1)(Value*, Value*, const std::string&) = &BinaryOperator::createOr;
    BinaryOperator* (*createOr2)(Value*, Value*, const std::string&, BasicBlock*) = &BinaryOperator::createOr;
    BinaryOperator* (*createOr3)(Value*, Value*, const std::string&, Instruction*) = &BinaryOperator::createOr;
    BinaryOperator* (*createXor1)(Value*, Value*, const std::string&) = &BinaryOperator::createXor;
    BinaryOperator* (*createXor2)(Value*, Value*, const std::string&, BasicBlock*) = &BinaryOperator::createXor;
    BinaryOperator* (*createXor3)(Value*, Value*, const std::string&, Instruction*) = &BinaryOperator::createXor;
    BinaryOperator* (*createSetEQ1)(Value*, Value*, const std::string&) = &BinaryOperator::createSetEQ;
    BinaryOperator* (*createSetEQ2)(Value*, Value*, const std::string&, BasicBlock*) = &BinaryOperator::createSetEQ;
    BinaryOperator* (*createSetEQ3)(Value*, Value*, const std::string&, Instruction*) = &BinaryOperator::createSetEQ;
    BinaryOperator* (*createSetNE1)(Value*, Value*, const std::string&) = &BinaryOperator::createSetNE;
    BinaryOperator* (*createSetNE2)(Value*, Value*, const std::string&, BasicBlock*) = &BinaryOperator::createSetNE;
    BinaryOperator* (*createSetNE3)(Value*, Value*, const std::string&, Instruction*) = &BinaryOperator::createSetNE;
    BinaryOperator* (*createSetLE1)(Value*, Value*, const std::string&) = &BinaryOperator::createSetLE;
    BinaryOperator* (*createSetLE2)(Value*, Value*, const std::string&, BasicBlock*) = &BinaryOperator::createSetLE;
    BinaryOperator* (*createSetLE3)(Value*, Value*, const std::string&, Instruction*) = &BinaryOperator::createSetLE;
    BinaryOperator* (*createSetGE1)(Value*, Value*, const std::string&) = &BinaryOperator::createSetGE;
    BinaryOperator* (*createSetGE2)(Value*, Value*, const std::string&, BasicBlock*) = &BinaryOperator::createSetGE;
    BinaryOperator* (*createSetGE3)(Value*, Value*, const std::string&, Instruction*) = &BinaryOperator::createSetGE;
    BinaryOperator* (*createSetLT1)(Value*, Value*, const std::string&) = &BinaryOperator::createSetLT;
    BinaryOperator* (*createSetLT2)(Value*, Value*, const std::string&, BasicBlock*) = &BinaryOperator::createSetLT;
    BinaryOperator* (*createSetLT3)(Value*, Value*, const std::string&, Instruction*) = &BinaryOperator::createSetLT;
    BinaryOperator* (*createSetGT1)(Value*, Value*, const std::string&) = &BinaryOperator::createSetGT;
    BinaryOperator* (*createSetGT2)(Value*, Value*, const std::string&, BasicBlock*) = &BinaryOperator::createSetGT;
    BinaryOperator* (*createSetGT3)(Value*, Value*, const std::string&, Instruction*) = &BinaryOperator::createSetGT;
    BinaryOperator* (*createNeg1)(Value*, const std::string&, Instruction*) = &BinaryOperator::createNeg;
    BinaryOperator* (*createNeg2)(Value*, const std::string&, BasicBlock*) = &BinaryOperator::createNeg;
    BinaryOperator* (*createNot1)(Value*, const std::string&, Instruction*) = &BinaryOperator::createNot;
    BinaryOperator* (*createNot2)(Value*, const std::string&, BasicBlock*) = &BinaryOperator::createNot;
    const Value* (*getNegArgument)(const BinaryOperator*) = &BinaryOperator::getNegArgument;
    const Value* (*getNotArgument)(const BinaryOperator*) = &BinaryOperator::getNotArgument;

    class_<BinaryOperator, boost::noncopyable, bases<Instruction> >("BinaryOperator", no_init)

        .def("create", create1,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<5, 0> >())
        .def("create", create2,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<5, 0> >())
        .staticmethod("create")
        .def("createAdd", createAdd1,
             return_value_policy<manage_new_object>())
        .def("createAdd", createAdd2,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .def("createAdd", createAdd3,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .staticmethod("createAdd")
        .def("createSub", createSub1,
             return_value_policy<manage_new_object>())
        .def("createSub", createSub2,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .def("createSub", createSub3,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .staticmethod("createSub")
        .def("createMul", createMul1,
             return_value_policy<manage_new_object>())
        .def("createMul", createMul2,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .def("createMul", createMul3,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .staticmethod("createMul")
        .def("createDiv", createDiv1,
             return_value_policy<manage_new_object>())
        .def("createDiv", createDiv2,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .def("createDiv", createDiv3,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .staticmethod("createDiv")
        .def("createRem", createRem1,
             return_value_policy<manage_new_object>())
        .def("createRem", createRem2,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .def("createRem", createRem3,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .staticmethod("createRem")
        .def("createAnd", createAnd1,
             return_value_policy<manage_new_object>())
        .def("createAnd", createAnd2,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .def("createAnd", createAnd3,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .staticmethod("createAnd")
        .def("createOr", createOr1,
             return_value_policy<manage_new_object>())
        .def("createOr", createOr2,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .def("createOr", createOr3,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .staticmethod("createOr")
        .def("createXor", createXor1,
             return_value_policy<manage_new_object>())
        .def("createXor", createXor2,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .def("createXor", createXor3,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .staticmethod("createXor")
        .def("createSetEQ", createSetEQ1,
            return_value_policy<manage_new_object>())
        .def("createSetEQ", createSetEQ2,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .def("createSetEQ", createSetEQ3,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .staticmethod("createSetEQ")
        .def("createSetNE", createSetNE1,
             return_value_policy<manage_new_object>())
        .def("createSetNE", createSetNE2,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .def("createSetNE", createSetNE3, 
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .staticmethod("createSetNE")
        .def("createSetLE", createSetLE1,
            return_value_policy<manage_new_object>())
        .def("createSetLE", createSetLE2,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .def("createSetLE", createSetLE3,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .staticmethod("createSetLE")
        .def("createSetGE", createSetGE1,
             return_value_policy<manage_new_object>())
        .def("createSetGE", createSetGE2,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .def("createSetGE", createSetGE3,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .staticmethod("createSetGE")
        .def("createSetLT", createSetLT1,
             return_value_policy<manage_new_object>())
        .def("createSetLT", createSetLT2,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .def("createSetLT", createSetLT3,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .staticmethod("createSetLT")
        .def("createSetGT", createSetGT1,
             return_value_policy<manage_new_object>())
        .def("createSetGT", createSetGT2,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .def("createSetGT", createSetGT3,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<4, 0> >())
        .staticmethod("createSetGT")
        .def("createNeg", createNeg1,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<3, 0> >())
        .def("createNeg", createNeg2,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<3, 0> >())
        .staticmethod("createNeg")
        .def("createNot", createNot1,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<3, 0> >())
        .def("createNot", createNot2,
             return_value_policy<reference_existing_object,
                 with_custodian_and_ward_postcall<3, 0> >())
        .staticmethod("createNot")
        .def("isNeg", &BinaryOperator::isNeg)
        .staticmethod("isNeg")
        .def("isNot", &BinaryOperator::isNot)
        .staticmethod("isNot")
        .def("getNegArgument", getNegArgument,
             return_internal_reference<>())
        .staticmethod("getNegArgument")
        .def("getNotArgument", getNotArgument,
             return_internal_reference<>())
        .staticmethod("getNotArgument")
        .def("getOpcode", &BinaryOperator::getOpcode)
        .def("clone", &BinaryOperator::clone,
             return_value_policy<manage_new_object>())
        .def("swapOperands", &BinaryOperator::swapOperands)
        .def("classof", classof_BinaryOperator)
        .def("classof", classof_Instruction)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_InstrTypes()
{
    wrap_TerminatorInst();
    wrap_BinaryOperator();
}

}} // namespace llvm::Python

// EOF
