/*
 * IntrinsicInst.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/IntrinsicInst.h>

using namespace boost::python;

namespace llvm { namespace Python {

void wrap_IntrinsicInst_()
{
    class_<IntrinsicInst, boost::noncopyable, bases<CallInst> >("IntrinsicInst", no_init)
        // result depends on arg
        .def("StripPointerCasts", &IntrinsicInst::StripPointerCasts,
             with_custodian_and_ward_postcall<1, 0,
                 return_arg<1> >())
        .staticmethod("StripPointerCasts")
        ;
}

void wrap_DbgInfoIntrinsic()
{
    // Name C++ overloads
    bool (*classof_DbgInfoIntrinsic)(const DbgInfoIntrinsic*) = &DbgInfoIntrinsic::classof;
    bool (*classof_CallInst)(const CallInst*) = &DbgInfoIntrinsic::classof;
    bool (*classof_Value)(const Value*) = &DbgInfoIntrinsic::classof;

    class_<DbgInfoIntrinsic, boost::noncopyable, bases<IntrinsicInst> >("DbgInfoIntrinsic", no_init)
        .def("getChain", &DbgInfoIntrinsic::getChain,
             return_internal_reference<>())
        .def("classof", classof_DbgInfoIntrinsic)
        .def("classof", classof_CallInst)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_DbgStopPointInst()
{
    // Name C++ overloads
    bool (*classof_DbgStopPointInst)(const DbgStopPointInst*) = &DbgStopPointInst::classof;
    bool (*classof_CallInst)(const CallInst*) = &DbgStopPointInst::classof;
    bool (*classof_Value)(const Value*) = &DbgStopPointInst::classof;

    class_<DbgStopPointInst, boost::noncopyable, bases<DbgInfoIntrinsic> >("DbgStopPointInst", no_init)
        .def("getLineNo", &DbgStopPointInst::getLineNo)
        .def("getColNo", &DbgStopPointInst::getColNo)
        .def("getContext", &DbgStopPointInst::getContext,
             return_internal_reference<>())
        .def("classof", classof_DbgStopPointInst)
        .def("classof", classof_CallInst)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_MemIntrinsic()
{
    // Name C++ overloads
    bool (*classof_MemIntrinsic)(const MemIntrinsic*) = &MemIntrinsic::classof;
    bool (*classof_CallInst)(const CallInst*) = &MemIntrinsic::classof;
    bool (*classof_Value)(const Value*) = &MemIntrinsic::classof;

    class_<MemIntrinsic, boost::noncopyable, bases<IntrinsicInst> >("MemIntrinsic", no_init)
        .def("getRawDest", &MemIntrinsic::getRawDest,
             return_internal_reference<>())
        .def("getLength", &MemIntrinsic::getLength,
             return_internal_reference<>())
        .def("getAlignment", &MemIntrinsic::getAlignment,
             return_internal_reference<>())
        .def("getDest", &MemIntrinsic::getDest,
             return_internal_reference<>())
        .def("setDest", &MemIntrinsic::setDest)
        .def("setLength", &MemIntrinsic::setLength)
        .def("setAlignment", &MemIntrinsic::setAlignment)
        .def("classof", classof_MemIntrinsic)
        .def("classof", classof_CallInst)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_MemCpyInst()
{
    // Name C++ overloads
    bool (*classof_MemCpyInst)(const MemCpyInst*) = &MemCpyInst::classof;
    bool (*classof_MemIntrinsic)(const MemIntrinsic*) = &MemCpyInst::classof;
    bool (*classof_CallInst)(const CallInst*) = &MemCpyInst::classof;
    bool (*classof_Value)(const Value*) = &MemCpyInst::classof;

    class_<MemCpyInst, boost::noncopyable, bases<MemIntrinsic> >("MemCpyInst", no_init)
        .def("getRawSource", &MemCpyInst::getRawSource,
             return_internal_reference<>())
        .def("getSource", &MemCpyInst::getSource,
             return_internal_reference<>())
        .def("setSource", &MemCpyInst::setSource)
        .def("classof", classof_MemCpyInst)
        .def("classof", classof_MemIntrinsic)
        .def("classof", classof_CallInst)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_MemMoveInst()
{
    // Name C++ overloads
    bool (*classof_MemMoveInst)(const MemMoveInst*) = &MemMoveInst::classof;
    bool (*classof_MemIntrinsic)(const MemIntrinsic*) = &MemMoveInst::classof;
    bool (*classof_CallInst)(const CallInst*) = &MemMoveInst::classof;
    bool (*classof_Value)(const Value*) = &MemMoveInst::classof;

    class_<MemMoveInst, boost::noncopyable, bases<MemIntrinsic> >("MemMoveInst", no_init)
        .def("getRawSource", &MemMoveInst::getRawSource,
             return_internal_reference<>())
        .def("getSource", &MemMoveInst::getSource,
             return_internal_reference<>())
        .def("setSource", &MemMoveInst::setSource)
        .def("classof", classof_MemMoveInst)
        .def("classof", classof_MemIntrinsic)
        .def("classof", classof_CallInst)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_MemSetInst()
{
    // Name C++ overloads
    bool (*classof_MemSetInst)(const MemSetInst*) = &MemSetInst::classof;
    bool (*classof_MemIntrinsic)(const MemIntrinsic*) = &MemSetInst::classof;
    bool (*classof_CallInst)(const CallInst*) = &MemSetInst::classof;
    bool (*classof_Value)(const Value*) = &MemSetInst::classof;

    class_<MemSetInst, boost::noncopyable, bases<MemIntrinsic> >("MemSetInst", no_init)
        .def("getValue", &MemSetInst::getValue,
             return_internal_reference<>())
        .def("setValue", &MemSetInst::setValue)
        .def("classof", classof_MemSetInst)
        .def("classof", classof_MemIntrinsic)
        .def("classof", classof_CallInst)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}

void wrap_IntrinsicInst()
{
    wrap_IntrinsicInst_();
    wrap_DbgInfoIntrinsic();
    wrap_DbgStopPointInst();
    wrap_MemIntrinsic();
    wrap_MemCpyInst();
    wrap_MemMoveInst();
    wrap_MemSetInst();
}

}} // namespace llvm::Python

// EOF

