/*
 * Module.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Function.h>
#include <llvm/Module.h>
#include <llvm/ModuleProvider.h>

using namespace boost::python;

namespace llvm { namespace Python {

// Module ownership transfer
struct ExistingModuleProviderWrap : public ExistingModuleProvider
{
    ExistingModuleProviderWrap(std::auto_ptr<Module> m)
        : ExistingModuleProvider(m.get())
    {
        m.release();
    }
};

void wrap_ModuleProvider()
{
    class_<ModuleProvider, boost::noncopyable>("ModuleProvider", no_init)
        .def("getModule", &ModuleProvider::getModule,
             return_internal_reference<>())
        .def("materializeFunction", &ModuleProvider::materializeFunction)
        .def("materializeModule", &ModuleProvider::materializeModule,
             return_internal_reference<>())
        .def("releaseModule", &ModuleProvider::releaseModule,
             return_value_policy<manage_new_object>())
        ;

    class_<ExistingModuleProviderWrap, boost::noncopyable, bases<ModuleProvider> >
        ("ExistingModuleProvider", init<std::auto_ptr<Module>& >()
         [with_custodian_and_ward<1, 2>()])
        .def("materializeFunction", &ExistingModuleProvider::materializeFunction)
        .def("materializeModule", &ExistingModuleProvider::materializeModule,
             return_internal_reference<>())
        ;
}

}} // namespace llvm::Python

// EOF
