/*
 * PassAnalysisSupport.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Pass.h>

#include "scitbx/container_conversions.h"

using namespace boost::python;
using namespace scitbx::boost_python::container_conversions;

namespace llvm { namespace Python {

void wrap_AnalysisUsage()
{
    class_<AnalysisUsage, boost::noncopyable>("AnalysisUsage")

         .def("addRequiredID", &AnalysisUsage::addRequiredID,
              return_self<>())
         //FIXME template
         //.def("addRequired", &AnalysisUsage::addRequired<PassClass>,
         //     return_self<>())
         //FIXME template
         //.def("addRequiredTransitive", &AnalysisUsage::addRequiredTransitive<PassClass>,
         //     return_self<>())
         .def("addPreservedID", &AnalysisUsage::addPreservedID,
              return_self<>())
         //FIXME template
         //.def("addPreserved", &AnalysisUsage::addPreserved<PassClass>,
         //     return_self<>())
         .def("setPreservesAll", &AnalysisUsage::setPreservesAll)
         .def("getPreservesAll", &AnalysisUsage::getPreservesAll)
         .def("setPreservesCFG", &AnalysisUsage::setPreservesCFG)
         .def("getRequiredSet", &AnalysisUsage::getRequiredSet,
              return_value_policy<copy_const_reference>())
         .def("getRequiredTransitiveSet", &AnalysisUsage::getRequiredTransitiveSet,
              return_value_policy<copy_const_reference>())
         .def("getPreservedSet", &AnalysisUsage::getPreservedSet,
              return_value_policy<copy_const_reference>())
        ;

    // Register tuple conversions for get* result vectors
    // -- already registered in PassSupport.cpp
    //to_python_converter<std::vector<AnalysisID>,
    //                    to_tuple<std::vector<AnalysisID> > >();
}

void wrap_AnalysisResolver()
{
    class_<AnalysisResolver, boost::noncopyable>("AnalysisResolver", no_init)
        .def("getAnalysisOrNullUp", &AnalysisResolver::getAnalysisOrNullUp,
             return_internal_reference<>())
        .def("getAnalysisOrNullDown", &AnalysisResolver::getAnalysisOrNullDown,
             return_internal_reference<>())
        .def("addPass", &AnalysisResolver::addPass)
        .def("getAnalysis", &AnalysisResolver::getAnalysis,
             return_internal_reference<>())
        .def("getAnalysisToUpdate", &AnalysisResolver::getAnalysisToUpdate,
             return_internal_reference<>())
        .def("getDepth", &AnalysisResolver::getDepth)
        .def("getNumContainedPasses", &AnalysisResolver::getNumContainedPasses)
        .def("getContainedPass", &AnalysisResolver::getContainedPass,
             return_internal_reference<>())
        .def("markPassUsed", &AnalysisResolver::markPassUsed)
        .def("startPass", &AnalysisResolver::startPass)
        .def("endPass", &AnalysisResolver::endPass)
        ;
}

void wrap_PassAnalysisSupport()
{
    wrap_AnalysisUsage();
    wrap_AnalysisResolver();
}

}} // namespace llvm::Python

// EOF
