/*
 * PassManager.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <memory>
#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/PassManager.h>
#include <llvm/Pass.h>
#include <llvm/Module.h>
#include <llvm/ModuleProvider.h>

using namespace boost::python;

namespace llvm { namespace Python {

/** Ownership transfer. */
void PassManager_add(PassManager& m, std::auto_ptr<Pass> p)
{
    m.add(p.get());
    p.release();
}

/** Ownership transfer. */
void FunctionPassManager_add_FunctionPass(FunctionPassManager& m, std::auto_ptr<FunctionPass> p)
{
    m.add(p.get());
    p.release();
}

/** Ownership transfer. */
void FunctionPassManager_add_ImmutablePass(FunctionPassManager& m, std::auto_ptr<ImmutablePass> p)
{
    m.add(p.get());
    p.release();
}

void wrap_PassManager()
{
    class_<PassManager, boost::noncopyable>("PassManager")
        .def("add", PassManager_add) // ownership transfer
        .def("run", &PassManager::run)
        ;

    class_<FunctionPassManager, boost::noncopyable>
        ("FunctionPassManager", init<ModuleProvider*>())
        .def("add", FunctionPassManager_add_FunctionPass) // ownership transfer
        .def("add", FunctionPassManager_add_ImmutablePass) // ownership transfer
        .def("run", &FunctionPassManager::run)
        ;
}


}} // namespace llvm::Python

// EOF
