/*
 * Type.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/Assembly/AsmAnnotationWriter.h>
#include <llvm/Type.h>
#include <llvm/Function.h>
#include <llvm/InstrTypes.h> // TerminatorInst
#include <llvm/SymbolTable.h>

using namespace boost::python;

namespace llvm { namespace Python {

void wrap_Type()
{
    scope TypeScope
        = class_<Type, boost::noncopyable>("Type", no_init)

        .def("print_", &Type::print)
        .def("dump", &Type::dump)
        .def("getTypeID", &Type::getTypeID)
        .def("getDescription", &Type::getDescription,
             return_value_policy<return_by_value>())
        .def("isSigned", &Type::isSigned)
        .def("isUnsigned", &Type::isUnsigned)
        .def("isInteger", &Type::isInteger)
        .def("isIntegral", &Type::isIntegral)
        .def("isFloatingPoint", &Type::isFloatingPoint)
        .def("isAbstract", &Type::isAbstract)
        .def("isLosslesslyConvertibleTo", &Type::isLosslesslyConvertibleTo)
        .def("isPrimitiveType", &Type::isPrimitiveType)
        .def("isDerivedType", &Type::isDerivedType)
        .def("isFirstClassType", &Type::isFirstClassType)
        .def("isSized", &Type::isSized)
        .def("getPrimitiveSize", &Type::getPrimitiveSize)
        .def("getUnsignedVersion", &Type::getUnsignedVersion,
             return_value_policy<reference_existing_object>())
        .def("getSignedVersion", &Type::getSignedVersion,
             return_value_policy<reference_existing_object>())
        .def("getForwardedType", &Type::getForwardedType,
             return_value_policy<reference_existing_object>())
        .add_property("subtypes",
                      range(&Type::subtype_begin,
                            &Type::subtype_end))
        //.def("subtype_begin", &Type::subtype_begin)
        //.def("subtype_end", &Type::subtype_end)
        .def("getContainedType", &Type::getContainedType,
             return_value_policy<reference_existing_object>())
        .def("getNumContainedTypes", &Type::getNumContainedTypes)
        .def("getPrimitiveType", &Type::getPrimitiveType,
             return_value_policy<reference_existing_object>())
        .staticmethod("getPrimitiveType")
        .add_static_property("VoidTy",
                             make_getter(*Type::VoidTy,
                                         return_value_policy<reference_existing_object>()))
        .add_static_property("BoolTy",
                             make_getter(*Type::BoolTy,
                                         return_value_policy<reference_existing_object>()))
        .add_static_property("SByteTy",
                             make_getter(*Type::SByteTy,
                                         return_value_policy<reference_existing_object>()))
        .add_static_property("UByteTy",
                             make_getter(*Type::UByteTy,
                                         return_value_policy<reference_existing_object>()))
        .add_static_property("ShortTy",
                             make_getter(*Type::ShortTy,
                                         return_value_policy<reference_existing_object>()))
        .add_static_property("UShortTy",
                             make_getter(*Type::UShortTy,
                                         return_value_policy<reference_existing_object>()))
        .add_static_property("IntTy",
                             make_getter(*Type::IntTy,
                                         return_value_policy<reference_existing_object>()))
        .add_static_property("UIntTy",
                             make_getter(*Type::UIntTy,
                                         return_value_policy<reference_existing_object>()))
        .add_static_property("LongTy",
                             make_getter(*Type::LongTy,
                                         return_value_policy<reference_existing_object>()))
        .add_static_property("ULongTy",
                             make_getter(*Type::ULongTy,
                                         return_value_policy<reference_existing_object>()))
        .add_static_property("FloatTy",
                             make_getter(*Type::FloatTy,
                                         return_value_policy<reference_existing_object>()))
        .add_static_property("DoubleTy",
                             make_getter(*Type::DoubleTy,
                                         return_value_policy<reference_existing_object>()))
        .add_static_property("LabelTy",
                             make_getter(*Type::LabelTy,
                                         return_value_policy<reference_existing_object>()))
        .def("classof", &Type::classof)
        .staticmethod("classof")
        // "llvm/Type.def"?
        .def("addAbstractTypeUser", &Type::addAbstractTypeUser)
        .def("removeAbstractTypeUser", &Type::removeAbstractTypeUser)
        .def("addRef", &Type::addRef)
        .def("dropRef", &Type::dropRef)
        .def("clearAllTypeMaps", &Type::clearAllTypeMaps)
        .staticmethod("clearAllTypeMaps")
        ;

    enum_<Type::TypeID>("TypeID")
        .value("VoidTyID", Type::VoidTyID)
        .value("BoolTyID", Type::BoolTyID)
        .value("UByteTyID", Type::UByteTyID)
        .value("SByteTyID", Type::SByteTyID)
        .value("UShortTyID", Type::UShortTyID)
        .value("ShortTyID", Type::ShortTyID)
        .value("UIntTyID", Type::UIntTyID)
        .value("IntTyID", Type::IntTyID)
        .value("ULongTyID", Type::ULongTyID)
        .value("LongTyID", Type::LongTyID)
        .value("FloatTyID", Type::FloatTyID)
        .value("DoubleTyID", Type::DoubleTyID)
        .value("LabelTyID", Type::LabelTyID)
        .value("FunctionTyID", Type::FunctionTyID)
        .value("StructTyID", Type::StructTyID)
        .value("ArrayTyID", Type::ArrayTyID)
        .value("PointerTyID", Type::PointerTyID)
        .value("OpaqueTyID", Type::OpaqueTyID)
        .value("PackedTyID", Type::PackedTyID)
        .value("NumTypeIDs", Type::NumTypeIDs)
        .value("LastPrimitiveTyID", Type::LastPrimitiveTyID)
        .value("FirstDerivedTyID", Type::FirstDerivedTyID)
        ;
}
}} // namespace llvm::Python

// EOF
