/*
 * User.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/User.h>
#include <llvm/SymbolTable.h>

using namespace boost::python;

namespace llvm { namespace Python {

void wrap_User()
{
    // Name C++ overloads
    User::const_op_iterator (User::*op_begin)() const = &User::op_begin;
    User::const_op_iterator (User::*op_end)() const = &User::op_end;
    bool (*classof_User)(const User*) = &User::classof;
    bool (*classof_Value)(const Value*) = &User::classof;

    class_<User, boost::noncopyable, bases<Value> >("User", no_init)
        // additional constructors
        //.def(init<const Type*, unsigned, const std::string>())

        .def("getOperand", &User::getOperand,
             return_internal_reference<>())
        .def("setOperand", &User::setOperand)
        .def("getNumOperands", &User::getNumOperands)
        .add_property("operands", range(op_begin, op_end))
        //.def("op_begin", op_begin)
        //.def("op_end", op_end)
        .def("dropAllReferences", &User::dropAllReferences)
        .def("replaceUsesOfWith", &User::replaceUsesOfWith)
        .def("classof", classof_User)
        .def("classof", classof_Value)
        .staticmethod("classof")
        ;
}
}} // namespace llvm::Python

// EOF
