/*
 * Value.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include <llvm/SymbolTable.h>
#include <llvm/Type.h>
#include <llvm/User.h>
#include <llvm/Value.h>

using namespace boost::python;

namespace llvm { namespace Python {

void wrap_Value()
{
    // Overloaded functions need some C++ preamble
    Value::use_const_iterator (Value::*use_begin)() const = &Value::use_begin;
    Value::use_const_iterator (Value::*use_end)() const = &Value::use_end;
    const User* (Value::*use_back)() const = &Value::use_back;

//    class_<Value, boost::noncopyable>("Value", init<const Type*, unsigned, optional<const std::string> >())
    class_<Value, boost::noncopyable>("Value", no_init)

        .def("dump", &Value::dump)
        //.def("print_", &Value::print)
        .def("getType", &Value::getType,
             return_internal_reference<>())
        .def("hasName", &Value::hasName)
        .def("getName", &Value::getName,
             return_value_policy<return_by_value>())
        .def("setName", &Value::setName)
        .def("replaceAllUsesWith", &Value::replaceAllUsesWith)
        .def("uncheckedReplaceAllUsesWith", &Value::uncheckedReplaceAllUsesWith)
        .def("use_empty", &Value::use_empty)
        //FIXME "error: invalid initialization of reference of type 'const llvm::Use&' from expression of type 'const llvm::User*'"
        //.add_property("uses", range(use_begin, use_end))
        //.def("use_begin", use_begin)
        //.def("use_end", use_end)
        .def("use_back", use_back,
             return_internal_reference<>())
        .def("hasOneUse", &Value::hasOneUse)
        .def("addUse", &Value::addUse)
        .def("getValueType", &Value::getValueType)
        .def("classof", &Value::classof)
        .staticmethod("classof")
        .def("getRawType", &Value::getRawType,
             return_internal_reference<>())
        // ostream writer operator<<
        //.def(str(self))
        ;
}
}} // namespace llvm::Python

// EOF
