#!/usr/bin/env python
# -*- coding: iso8859-1 -*-
#
# fibonacci.py: Python bindings for the LLVM compiler infrastructure
# Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
# Distributed under the Boost Software License, Version 1.0. (See
# accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)
#

import llvm, sys

def create_fibonacci_function(m):
    fib_f = m.getOrInsertFunction("fib", llvm.Type.IntTy, (llvm.Type.IntTy, ))
    bb = llvm.BasicBlock.init("EntryBlock", fib_f)
    one = llvm.ConstantSInt.get(llvm.Type.IntTy, 1)
    two = llvm.ConstantSInt.get(llvm.Type.IntTy, 2)

    arg_x = fib_f.afront()
    arg_x.setName("AnArg")

    ret_bb = llvm.BasicBlock.init("return", fib_f)
    recurse_bb = llvm.BasicBlock.init("recurse", fib_f)
    cond_inst = llvm.BinaryOperator.createSetLE(arg_x, two, "cond", bb)
    llvm.BranchInst.init(ret_bb, recurse_bb, cond_inst, bb)
    llvm.ReturnInst.init(one, ret_bb)
    sub = llvm.BinaryOperator.createSub(arg_x, one, "arg", recurse_bb)
    call_fib_x1 = llvm.CallInst.init(fib_f, sub, "fibx1", recurse_bb)
    sub2 = llvm.BinaryOperator.createSub(arg_x, two, "arg", recurse_bb)
    call_fib_x2 = llvm.CallInst.init(fib_f, sub2, "fibx2", recurse_bb)
    sum = llvm.BinaryOperator.createAdd(call_fib_x1, call_fib_x2, "addresult", recurse_bb)
    llvm.ReturnInst.init(sum, recurse_bb)
    return fib_f

m = llvm.Module("test")
f = create_fibonacci_function(m)
assert not llvm.verifyModule(m)

mp = llvm.ExistingModuleProvider(m)
jit = llvm.ExecutionEngine.create(mp, False)

# print function
f.dump()

try:
    n = int(sys.argv[1])
except:
    n = 24

print "starting fibonacci(%d) with JIT..." % n;

arg = llvm.GenericValue()
arg.IntVal = n
gv = jit.runFunction(f, (arg, ))

print "Result: %d" % gv.IntVal
