/* -*- c++ -*-
 * inst_init.h: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef INST_INIT_H
#define INST_INIT_H

// assume "using namespace boost::python" is in effect

#define INST_INIT_DEF1(I,A1)                InstInitFn1<I,A1>, InstInitPolicy1<A1>()
#define INST_INIT_DEF2(I,A1,A2)             InstInitFn2<I,A1,A2>, InstInitPolicy2<A1,A2>()
#define INST_INIT_DEF3(I,A1,A2,A3)          InstInitFn3<I,A1,A2,A3>, InstInitPolicy3<A1,A2,A3>()
#define INST_INIT_DEF4(I,A1,A2,A3,A4)       InstInitFn4<I,A1,A2,A3,A4>, InstInitPolicy4<A1,A2,A3,A4>()
#define INST_INIT_DEF5(I,A1,A2,A3,A4,A5)    InstInitFn5<I,A1,A2,A3,A4,A5>, InstInitPolicy5<A1,A2,A3,A4,A5>()
#define INST_INIT_DEF6(I,A1,A2,A3,A4,A5,A6) InstInitFn6<I,A1,A2,A3,A4,A5,A6>, InstInitPolicy6<A1,A2,A3,A4,A5,A6>()

namespace llvm { namespace Python {

// Thin constructor wrappers
template<class I, class A1>
I* InstInitFn1(A1 a1) {
    return new I(a1);
}

template<class I, class A1, class A2>
I* InstInitFn2(A1 a1, A2 a2) {
    return new I(a1, a2);
}

template<class I, class A1, class A2, class A3>
I* InstInitFn3(A1 a1, A2 a2, A3 a3) {
    return new I(a1, a2, a3);
}

template<class I, class A1, class A2, class A3, class A4>
I* InstInitFn4(A1 a1, A2 a2, A3 a3, A4 a4) {
    return new I(a1, a2, a3, a4);
}

template<class I, class A1, class A2, class A3, class A4, class A5>
I* InstInitFn5(A1 a1, A2 a2, A3 a3, A4 a4, A5 a5) {
    return new I(a1, a2, a3, a4, a5);
}

template<class I, class A1, class A2, class A3, class A4, class A5, class A6>
I* InstInitFn6(A1 a1, A2 a2, A3 a3, A4 a4, A5 a5, A6 a6) {
    return new I(a1, a2, a3, a4, a5, a6);
}

// Generally, assume ownership from Instr constructors
// equivalent to return_value_policy<manage_new_object>
template<class A1>
struct InstInitPolicy1 : default_call_policies
{
    typedef manage_new_object result_converter;
};

template<class A1, class A2>
struct InstInitPolicy2 : default_call_policies
{
    typedef manage_new_object result_converter;
};

template<class A1, class A2, class A3>
struct InstInitPolicy3 : default_call_policies
{
    typedef manage_new_object result_converter;
};

template<class A1, class A2, class A3, class A4>
struct InstInitPolicy4 : default_call_policies
{
    typedef manage_new_object result_converter;
};

template<class A1, class A2, class A3, class A4, class A5>
struct InstInitPolicy5 : default_call_policies
{
    typedef manage_new_object result_converter;
};

template<class A1, class A2, class A3, class A4, class A5, class A6>
struct InstInitPolicy6 : default_call_policies
{
    typedef manage_new_object result_converter;
};

// Transfer ownership when last argument is BasicBlock*
// eq. return_value_policy<reference_existing_object,
//         with_custodian_and_ward_postcall<N+1, 0> >
template<>
struct InstInitPolicy1<BasicBlock*> : with_custodian_and_ward_postcall<1, 0>
{
    typedef reference_existing_object result_converter;
};

template<class A1>
struct InstInitPolicy2<A1, BasicBlock*> : with_custodian_and_ward_postcall<2, 0>
{
    typedef reference_existing_object result_converter;
};

template<class A1, class A2>
struct InstInitPolicy3<A1, A2, BasicBlock*> : with_custodian_and_ward_postcall<3, 0>
{
    typedef reference_existing_object result_converter;
};

template<class A1, class A2, class A3>
struct InstInitPolicy4<A1, A2, A3, BasicBlock*> : with_custodian_and_ward_postcall<4, 0>
{
    typedef reference_existing_object result_converter;
};

template<class A1, class A2, class A3, class A4>
struct InstInitPolicy5<A1, A2, A3, A4, BasicBlock*> : with_custodian_and_ward_postcall<5, 0>
{
    typedef reference_existing_object result_converter;
};

template<class A1, class A2, class A3, class A4, class A5>
struct InstInitPolicy6<A1, A2, A3, A4, A5, BasicBlock*> : with_custodian_and_ward_postcall<6, 0>
{
    typedef reference_existing_object result_converter;
};

// Transfer ownership when last argument is Instruction*
// eq. return_value_policy<reference_existing_object,
//         with_custodian_and_ward_postcall<N+1, 0> >
template<class A1>
struct InstInitPolicy2<A1, Instruction*> : with_custodian_and_ward_postcall<2, 0>
{
    typedef reference_existing_object result_converter;
};

template<class A1, class A2>
struct InstInitPolicy3<A1, A2, Instruction*> : with_custodian_and_ward_postcall<3, 0>
{
    typedef reference_existing_object result_converter;
};

template<class A1, class A2, class A3>
struct InstInitPolicy4<A1, A2, A3, Instruction*> : with_custodian_and_ward_postcall<4, 0>
{
    typedef reference_existing_object result_converter;
};

template<class A1, class A2, class A3, class A4>
struct InstInitPolicy5<A1, A2, A3, A4, Instruction*> : with_custodian_and_ward_postcall<5, 0>
{
    typedef reference_existing_object result_converter;
};

template<class A1, class A2, class A3, class A4, class A5>
struct InstInitPolicy6<A1, A2, A3, A4, A5, Instruction*> : with_custodian_and_ward_postcall<6, 0>
{
    typedef reference_existing_object result_converter;
};

}} // namespace llvm::Python

#endif

// EOF
