/*
 * llvm.cpp: Python bindings for the LLVM compiler infrastructure
 * Copyright (c) 2004, 2005 Jarno Seppänen <firstname.lastname@iki.fi>.
 * Distributed under the Boost Software License, Version 1.0. (See
 * accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <boost/python.hpp>
#include <iostream>

using namespace boost::python;

namespace llvm { namespace Python {

void wrap_AbstractTypeUser();
void wrap_Analysis_Verifier();
void wrap_Annotation();
void wrap_Argument();
void wrap_BasicBlock();
void wrap_Bytecode_Reader();
void wrap_Bytecode_Writer();
void wrap_CallGraphSCCPass();
void wrap_Constant();
void wrap_Constants();
void wrap_DerivedTypes();
void wrap_ExecutionEngine();
void wrap_Function();
void wrap_GenericValue();
void wrap_GlobalValue();
void wrap_GlobalVariable();
void wrap_InstrTypes();
void wrap_Instruction();
void wrap_Instructions();
void wrap_IntrinsicInst();
void wrap_Intrinsics(object module);
void wrap_Linker();
void wrap_Module();
void wrap_ModuleProvider();
void wrap_Pass();
void wrap_PassAnalysisSupport();
void wrap_PassManager();
void wrap_PassSupport();
void wrap_SymbolTable();
void wrap_Type();
void wrap_Use();
void wrap_User();
void wrap_Value();
void wrap_System_Path(object module);

BOOST_PYTHON_MODULE(llvm)
{
    // Create sub-namespaces
    object IntrinsicModule(handle<>(PyModule_New((char*)"llvm.Intrinsic")));
    object sysModule(handle<>(PyModule_New((char*)"llvm.sys")));
    scope().attr("Intrinsic") = IntrinsicModule;
    scope().attr("sys") = sysModule;

    // wrapping order is significant (bases first)

    // VMCore
    wrap_Value();
    wrap_Type();
    wrap_User();
    wrap_AbstractTypeUser();
    wrap_BasicBlock();
    wrap_Constant();
    wrap_Constants();
    wrap_DerivedTypes();
    wrap_Module();
    wrap_Linker();
    wrap_ModuleProvider();
    wrap_Use();
    wrap_Argument();
    wrap_Annotation();
    wrap_GlobalValue();
    wrap_GlobalVariable();
    wrap_Function();
    wrap_Instruction();
    wrap_InstrTypes();
    wrap_Instructions();
    wrap_IntrinsicInst();
    wrap_Intrinsics(IntrinsicModule);
    wrap_GenericValue();
    wrap_ExecutionEngine();
    wrap_Pass();
    wrap_PassAnalysisSupport();
    wrap_PassManager();
    wrap_PassSupport();
    wrap_CallGraphSCCPass();
    wrap_SymbolTable();

    // Analysis
    wrap_Analysis_Verifier();

    // Bytecode
    wrap_Bytecode_Reader();
    wrap_Bytecode_Writer();

    // System
    wrap_System_Path(sysModule);

}

}} // namespace llvm::Python

// EOF
